local tablet = AceLibrary("Tablet-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
local deformat = AceLibrary("Deformat-2.0")
local quixote = AceLibrary("Quixote-1.0")

wowheadComment = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0", "AceHook-2.1")

local defaults = {
	debug = false,
	width = 900,
	tablet_data = {detached=true, },
}

local isVisableQuest = { }

-- Hooks the quest log click on Player Enter World and loads faction database
function wowheadComment:OnStartup()
     -- create the alternate function
     function wowheadComment:QuestClick(a1, ...)
        -- hook in my own junk if SHIFT is held down
        if IsAltKeyDown() and not IsShiftKeyDown() then
           local questName = this:GetText();
           --DEFAULT_CHAT_FRAME:AddMessage(questName);         
           local _, _, _, _, _, _, _, questid = quixote:GetQuest(questName)        
           if questid then
             wowheadComment:RefreshTablet(questid);
            -- wowheadComment:displayComments(questid);
           elseif questName then
             questName = this:GetText():trim();    
             _, _, _, _, _, _, _, questid = quixote:GetQuest(questName)  
             if questid then
              wowheadComment:RefreshTablet(questid);
             end
           end
           return Wowhead_QuestLogTitleButton_OnClick(a1, ...);
        else
          -- DEFAULT_CHAT_FRAME:AddMessage("Debug: ALT not pressed.");
           return Wowhead_QuestLogTitleButton_OnClick(a1, ...);
        end
     end
     -- here is where the hook for QuestClick takes place
     Wowhead_QuestLogTitleButton_OnClick = QuestLogTitleButton_OnClick;
     QuestLogTitleButton_OnClick = wowheadComment.QuestClick;
     -- Load on demand databases
     wowheadComment:LoadOnDemandFaction()
end

-- Loads the addon stub for the appropriate faction database
function wowheadComment:LoadOnDemandFaction()
        local my_database_faction = UnitFactionGroup("player");
        local wow_database_loaded = nil
        if my_database_faction == "Horde" then
           wow_database_loaded = LoadAddOn("Wowhead_Horde_Stub")         
        elseif my_database_faction == "Alliance" then
           wow_database_loaded = LoadAddOn("Wowhead_Alliance_Stub") 
        end 
        if not wow_database_loaded then
           DEFAULT_CHAT_FRAME:AddMessage("Wowhead Error: Database stub not found.");
        end
end


local function sortedpairs(t,comparator)
	local sortedKeys = {};
	table.foreach(t, function(k,v) table.insert(sortedKeys,k) end);
	table.sort(sortedKeys,comparator);
	local i = 0;
	local function _f(_s,_v)
		i = i + 1;
		local k = sortedKeys[i];
		if (k) then
			return k,t[k];
		end
	end
	return _f,nil,nil;
end


-- This creates the tablet
function wowheadComment:RefreshTablet(questID)
  local title, qData = self:getComments(questID)
  if not isVisableQuest[title] then
    if tablet:IsRegistered(title) then tablet:Unregister(title) end
    if not tablet:IsRegistered(title) then
      tablet:Register(
	 title,
	 'children', function() wowheadComment:TabletContents(questID, title, qData) end,
     --    'parent', "QuestLogFrame",
         'point', function() return "BOTTOMRIGHT", "TOPLEFT" end,
         'detachedData', {detached=true },
         'cantAttach', true,
   	 'dontHook', true, 
         'showTitleWhenDetached', true,	  
	 'clickable', true,
	 'hideWhenEmpty', true,
			"menu", function()
				dewdrop:AddLine(
					"text", "Close Wowhead Comments",
					"tooltipTitle", "Close window",
					"tooltipText", "Closes the  query window.",
					"func", function()
						isVisableQuest[title] = false
                                                tablet:Attach(title)
						dewdrop:Close()
					end)
                               end
         )
    end
    if tablet:IsAttached(title) then tablet:Detach(title) end
    tablet:Open(title)
    tablet:Refresh(title)
  end
end


local aComments = {}

-- Core function for displaying comments
function wowheadComment:TabletContents(questID, title, qData)
  aComments = {}
  if ( qData and qData["comments"] ) then
  	  aComments = qData["comments"]
          tablet:SetTitle("                                                   "..title.."                                                   ")
          local cat = tablet:AddCategory('id',title,
             'child_size', 12,
             'child_textR', 1,
             'child_textG', 1,
             'child_textB', 1)
	    for k, v in sortedpairs(aComments) do 
	      local c = string.gsub(string.gsub(v, "(%d+)[%s,.|]+(%d+)", "|cffffd700 %1 %2 |r"),"(<http:.->)", "|cff0101ee %1 |r")
              if k and v and c then
	        local wcomment = "|cffaa4444" ..k .. "|r.  " .. c
                cat:AddLine('text', wcomment,
                 'wrap', true)
              end
	    end
  isVisableQuest[title] = true
  else
      tablet:SetTitle("                                                   "..title.."                                                   ")
      local cat = tablet:AddCategory('id',title,
         'child_size', 12,
         'child_textR', 1,
         'child_textG', 1,
         'child_textB', 1)
      cat:AddLine('text', "No Wowhead Comments. You can add some at: |cff0101eehttp://www.wowhead.com|r") 
  isVisableQuest[title] = true
  end 
end


-- Alternate to using tablet while if figure out how to create tablets
function wowheadComment:displayComments(questID)
  local title, qData = self:getComments(questID)
  aComments = {}
  if ( qData and qData["comments"] ) then
  	  aComments = qData["comments"]
	  DEFAULT_CHAT_FRAME:AddMessage(title)
	    for k, v in sortedpairs(aComments) do 
	      local c = string.gsub(string.gsub(v, "(%d+)[%s,.|]+(%d+)", "|cffffd700 %1 %2 |r"),"(<http:.->)", "|cff0101ee %1 |r")
	      DEFAULT_CHAT_FRAME:AddMessage("|cffaa4444" ..k .. "|r.  " .. c)
	    end
  end 
end

function wowheadComment:OnInitialize()
	self.options = {
		type="group",
		args = {
			debug = {
				name =	"Debug", type = "toggle",
				desc =	"Debug on/off",
				get =	function()
							return self.db.profile.debug
						end,
						set = function(v)
							self.db.profile.debug = v
						end,
			},
			width = {
				name =	"Frame width", type = "text",
				desc =	"Set frame width",
				usage = "width",
				get =	function()
							return self.db.profile.width
						end,
						set = function(v)
							self.db.profile.width = v
						end,
			},
		},
	}
	
	-- Register everything
	self:RegisterDB("wowheadCommentDB")
	self:RegisterDefaults('profile', defaults)
        self:RegisterEvent("PLAYER_ENTERING_WORLD","OnStartup",true)
	DEFAULT_CHAT_FRAME:AddMessage("|cffffd700Wowhead Quest Comments: |r Open questlog and ALT-CLICK the quest to open Wowhead Comments. To close, right-click Comments.")
end